package ds

case class Point(x: Int, y: Int)

case class Boundary(bottomLeft: Point, topRight: Point):
  /** Check whether this boundary is empty. */
  def isEmpty: Boolean =
    !(bottomLeft.x < topRight.x && bottomLeft.y < topRight.y)

  /** Check whether this boundary contains a point. */
  def contains(p: Point): Boolean =
    (p.x >= bottomLeft.x && p.x < topRight.x) && (p.y >= bottomLeft.y && p.y < topRight.y)

  /** Intersect this boundary with another. */
  def intersect(other: Boundary): Boundary =
    val startX = bottomLeft.x `max` other.bottomLeft.x
    val endX = topRight.x `min` other.topRight.x
    val startY = bottomLeft.y `max` other.bottomLeft.y
    val endY = topRight.y `min` other.topRight.y
    Boundary(Point(startX, startY), Point(endX, endY))

trait QuadTreeOps[T]:
  /** Create an empty quadtree on a `boundary` with a `capacity`. */
  def empty(boundary: Boundary, capacity: Int): T

  /** Insert a point into a quadtree. */
  def insert(tree: T, p: Point): T

  /** Return the points that fall within a boundary. */
  def query(tree: T, boundary: Boundary): List[Point]

// hint: define tree nodes as a enum?
type QuadTree = Nothing

object QuadTreeImpl extends QuadTreeOps[QuadTree]:
  def empty(boundary: Boundary, capacity: Int): QuadTree = ???
  def insert(tree: QuadTree, p: Point): QuadTree = ???
  def query(tree: QuadTree, boundary: Boundary): List[Point] = ???

/** Syntax for quadtrees. */
object QuadTreeSyntax:
  extension [X](tree: X)(using instance: QuadTreeOps[X])
    def insert(p: Point): X = instance.insert(tree, p)
    def query(boundary: Boundary): List[Point] = instance.query(tree, boundary)

  given quadTreeOps: QuadTreeOps[QuadTree] = QuadTreeImpl
